//
// ********************************************************************
// *                                                                  *
// *   Test Case for Geant4 Simulation:                               *
// *     Platinum Target 3cm dia  6cm long                            *
// *     30Gev Proton Beam  pointed at center of target               *
// *     Detect particle type and count at 16 degree angle            *
// *     at solid angle of 5 degree a distance of 14.5cm              *
// *     from beam contact point on target                            *
// *                                                                  *
// ********************************************************************
//
//
#include <stdlib.h>

//ROOT LIBRARIES
#include "TRandom.h"
#include "TFile.h"
#include "hList.hh"
#include "TTree.h"
#include "tripTreeStructs.hh"

#include "myDetectorConstruction.hh"
#include "myPhysicsList.hh"
#include "myPrimaryGeneratorAction.hh"
#include "myRunAction.hh"
#include "myEventAction.hh"
#include "mySteppingAction.hh"
#include "mySteppingVerbose.hh"


#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"
#include "G4SystemOfUnits.hh"


#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#endif


#ifdef G4UI_USE
#include "G4UIExecutive.hh"
#endif


struct mcDefaults_t {
char outFileName[80];
int nToGen,visModeUse;
double thickness;
double beamMomentum;
};


//FUNCTION PROTOTYPES
void printUsage(); // print usage
mcDefaults_t readMcDefaults(); //Read the default settings from mcDefaults.conf 


// ********************************************************************
//--> Histos    

mcHistos_t myHistos;

int main(int argc,char** argv)
{
  char outFileName[80];
  int nToGen,visModeUse;
  double thickness;
  double beamMomentum;
  char *argptr;
  //Read the defaults
  mcDefaults_t mcDefaults = readMcDefaults(); 
  
  //Set the defaults
  nToGen = mcDefaults.nToGen;
  thickness = mcDefaults.thickness;
  beamMomentum = mcDefaults.beamMomentum;
  visModeUse = mcDefaults.visModeUse;
  sprintf(outFileName,mcDefaults.outFileName);
  for (int i=1; i<argc; i++) {
    argptr = argv[i];
    if (*argptr == '-') {
      argptr++;
      switch (*argptr) {
      case 'h':
        printUsage();
        break;
      case 't':
        thickness = atof(++argptr);
        break;
      case 'p':
        beamMomentum = atof(++argptr);
        break;
      case 'n':
        nToGen = atoi(++argptr);
        break;
      case 'v':
        visModeUse = atoi(++argptr);
        break;
      case 'o':
        strcpy(outFileName,++argptr);
        break;
      default:
        fprintf(stderr,"Unrecognized argument: [-%s]\n\n",argptr);
        printUsage();
        break;
      }
    }
  }

  TTree *outTree = new TTree("outTree","crystalArray");
  crystalArray_t crystalArray; //crystalArray_t defined in outTree.h


  outTree->Branch("x",&crystalArray.x,"x/D");
  outTree->Branch("y",&crystalArray.y,"y/D");
  outTree->Branch("energy",&crystalArray.energy,"energy/D");

  outTree->Branch("row","std::vector<Int_t>",&crystalArray.row);
  outTree->Branch("column","std::vector<Int_t>",&crystalArray.column);
  outTree->Branch("eDep","std::vector<Double_t>",&crystalArray.eDep);
  
  myHistos.hEDep = new TH2D("hEDep","",10,-10.0,10.0,10,-10.0,10.0); 
  myHistos.hHitBlankYvsX = new TH2D("hHitBlankYvsX","",20,-10.0,10.0,4000,-200.0,200.0); 
  myHistos.hHitBlankYvsKe = new TH2D("hHitBlankYvsKe","",2000,0.0,20.0,4000,-200.0,200.0); 

  TFile *fout = new TFile(outFileName,"RECREATE");  //DEFINE OUTPUT ROOT FILE 


  // Run manager
  //
  G4RunManager * runManager = new G4RunManager;

  // User Initialization classes (mandatory)
  //

  myDetectorConstruction* detector = new myDetectorConstruction;
  //detector->SetConverterThickness(thickness*um);
  runManager->SetUserInitialization(detector);
  G4VUserPhysicsList* physics = new myPhysicsList;
  runManager->SetUserInitialization(physics);

  // User Action classes
  //

  myPrimaryGeneratorAction* gen_action = new myPrimaryGeneratorAction;
  runManager->SetUserAction(gen_action);
  gen_action->SetBeamMomentum(beamMomentum*MeV);
  //G4UserEventAction* event_action = new myEventAction;
  myEventAction* event_action = new myEventAction(gen_action);

  event_action->SetOutTree(outTree);
  event_action->SetOutTreeStruct(&crystalArray);

  runManager->SetUserAction(event_action);
  mySteppingAction* stepping_action =
    new mySteppingAction(detector, event_action, gen_action);
  runManager->SetUserAction(stepping_action);
 // Initialize G4 kernel
 // 
  G4UImanager* UI = G4UImanager::GetUIpointer();
  UI->ApplyCommand("/tracking/verbose 1"); 
  //UI->ApplyCommand("/tracking/verbose 0"); 
  //UI->ApplyCommand("/event/verbose 0");
  //UI->ApplyCommand("/event/verbose 1"); 
 runManager->Initialize();
 G4UImanager* UImanager = G4UImanager::GetUIpointer();

 if (visModeUse != 1)   // batch mode
   {
  G4cout<<"IN vismodeuse !=1"<<G4endl;
     G4String command = "/control/execute ";

  //G4String fileName = argv[1];

     runManager->BeamOn(nToGen); //Number of events to generate

   }
 else
   {  // interactive mode : define UI session

#ifdef G4VIS_USE
     //Initialize visualization                                                   
       G4VisManager* visManager = new G4VisExecutive;
     visManager->Initialize();
#endif
     
#ifdef G4UI_USE
     G4UIExecutive* ui = new G4UIExecutive(argc, argv);
#ifdef G4VIS_USE
     //     UImanager->ApplyCommand("/control/execute ../crystalGrid/vis.mac");
     UImanager->ApplyCommand("/control/execute ./vis.mac");
#endif
     //if (ui->IsGUI())
       //UImanager->ApplyCommand("/control/execute visTutor/gui.mac");
     ui->SessionStart();
     delete ui;
#endif
   }
 
 //END NEW

 delete runManager;
 

 fout->cd();
 myHistos.hEDep->Write();
 myHistos.hHitBlankYvsX->Write();
 myHistos.hHitBlankYvsKe->Write();
 outTree->Write();


 fout->Close();


 return 0;
}

////////////////////////////////////////
void printUsage(){
  fprintf(stderr,"\nSWITCHES:\n");
  fprintf(stderr,"-h\tPrint this message\n");
  fprintf(stderr,"-n<arg>\tNumber of events to generate\n");
  fprintf(stderr,"-t<arg>\tThickness in microns\n");
  fprintf(stderr,"-p<arg>\tBeam Momentum in MeV\n");
  fprintf(stderr,"-o<arg>\tOutFile name\n");
  fprintf(stderr,"-v<arg>\tVisualizaton mode if set to 1\n");


  G4cout<<""<<G4endl;
  G4cout<<"The above switches overide the default setting."<<G4endl;
  G4cout<<"The default settings are found in the file mcDefaults.conf"<<G4endl;
  G4cout<<"The user should modify the vsDefault.conf file to suit their taste"<<G4endl;


  G4cout<<""<<G4endl;
  mcDefaults_t mcDefaults = readMcDefaults(); //Read the defaults

  G4cout<<"The current default operation is equivalent to the command:"<<G4endl;
  G4cout<<"my"
	<<" -n"<<mcDefaults.nToGen
	<<" -t"<<mcDefaults.thickness
	<<" -p"<<mcDefaults.beamMomentum
	<<" -o"<<mcDefaults.outFileName
	<<" -v"<<mcDefaults.visModeUse
	<<"\n"<<G4endl;

  exit(0);
}
////////////////////////////////////////
mcDefaults_t readMcDefaults(){
  mcDefaults_t mcDefaults;
  char *fToRead;
  char parType[80];
  fToRead = "./mcDefaults.conf";
  std::string line;
  std::stringstream osl(line);
  std::string temp;
  std::ifstream myfile (fToRead);
  //ifstream myfile ("./mcDefaults.conf");
  int varI=0,emptyLine;
  double varD = 0.0;
  char varC[80];
  if (myfile.is_open())
    {

      //cout<<"\nReading default parameters from mcDefaults.conf\n"<<endl;   
      while ( myfile.good() )
        {
          int i = 0;
          getline(myfile,line);
	  std::stringstream osl(line);
          emptyLine = 1;
          while (osl >> temp) {
            emptyLine = 0;
            if (i==0){
              sprintf(parType,temp.c_str());
            } else {
              if (i==1) varI = atoi(temp.c_str());
              if (i==1) varD = atof(temp.c_str());
              if (i==1) sprintf(varC,temp.c_str());
            }
            i++;
          }
          if (emptyLine==0){
            if (strcmp(parType,"outFileName") == 0) sprintf(mcDefaults.outFileName,varC);
            if (strcmp(parType,"nToGen") == 0) mcDefaults.nToGen = varI;
            if (strcmp(parType,"thickness") == 0) mcDefaults.thickness = varD;
	    if (strcmp(parType,"beamMomentum") == 0) mcDefaults.beamMomentum = varD;
	    if (strcmp(parType,"visModeUse") == 0) mcDefaults.visModeUse = varI;
          }
        }
      //G4cout<<"\nFinished reading default parameters from mcDefaults.conf\n"<<G4endl;   
      myfile.close();
    }
  else G4cout << "!!!!!!!!!!!Unable to open mcDefaults.conf!!!!!!!!!!";
  return mcDefaults;
}
